@@ -1,9 +1,11 @@
 from email.message import EmailMessage
 from email.utils import localtime
+import json
 import logging
 from logging import StreamHandler
 from logging.handlers import TimedRotatingFileHandler
 import os
+import signal
 import smtplib
 import socket
 import sys
@@ -14,14 +16,25 @@ import traceback
 logging_setup = None
 
 
-def setup_logging(app_name=None):
+def setup_logging(app_name=None, default_level=logging.INFO, signal=None):
     global logging_setup
-    logging_setup = LoggingSetup(app_name)
+    logging_setup = LoggingSetup(
+        app_name=app_name,
+        default_level=default_level,
+        signal=signal,
+    )
     logging_setup.setup()
 
 
 class LoggingSetup:
-    def __init__(self, app_name=None):
+    logger = None
+    stdout_logger = None
+    stderr_logger = None
+    file_logger = None
+    syslog_logger = None
+    smtp_logger = None
+
+    def __init__(self, app_name=None, default_level=logging.INFO, signal=None):
         self.app_name = app_name
 
         self.debug = False
@@ -31,6 +44,7 @@ class LoggingSetup:
         self.file_log_count = 14
         self.file_log_when_rotate = "w0"
         self.file_log_rotate_interval = 1
+        self.file_log_encoding = "utf-8"
 
         self.syslog = False
         self.syslog_tag = None
@@ -41,7 +55,7 @@ class LoggingSetup:
         self.smtp_log_sender = None
         self.smtp_log_recipient = "root"
 
-        self.default_level = logging.INFO
+        self.default_level = default_level
         self.default_stderr_level = logging.WARNING
         self.default_file_level = self.default_level
         self.default_syslog_level = self.default_level
@@ -60,6 +74,9 @@ class LoggingSetup:
             ": %(levelname)s - %(message)s",
         )
 
+        self.toggle_level_signal = signal
+        self.toggle_level = self.default_level
+
     def setup(self):
         if self.app_name is None:
             try:
@@ -69,25 +86,25 @@ class LoggingSetup:
             except AttributeError:
                 self.app_name = os.path.basename(os.getcwd())
 
-        logger = logging.getLogger()
-        logger.setLevel(logging.NOTSET)
+        self.logger = logging.getLogger()
+        self.logger.setLevel(logging.NOTSET)
 
         stream_formatter = self.formatter
         if self.systemd_log or "SYSTEMD_LOG" in os.environ:
             stream_formatter = self.formatter_without_timestamp
 
-        handler = StreamHandler(sys.stderr)
-        handler.setLevel(self.default_stderr_level)
-        handler.setFormatter(stream_formatter)
-        logger.addHandler(handler)
+        self.stderr_logger = StreamHandler(sys.stderr)
+        self.stderr_logger.setLevel(self.default_stderr_level)
+        self.stderr_logger.setFormatter(stream_formatter)
+        self.logger.addHandler(self.stderr_logger)
 
-        handler = StreamHandler(sys.stdout)
-        handler.setLevel(self.default_level)
+        self.stdout_logger = StreamHandler(sys.stdout)
+        self.stdout_logger.setLevel(self.default_level)
         if self.debug or "DEBUG_LOG" in os.environ:
-            handler.setLevel(logging.DEBUG)
-        handler.addFilter(LessThanFilter(self.default_stderr_level))
-        handler.setFormatter(stream_formatter)
-        logger.addHandler(handler)
+            self.stdout_logger.setLevel(logging.DEBUG)
+        self.stdout_logger.addFilter(LessThanFilter(self.default_stderr_level))
+        self.stdout_logger.setFormatter(stream_formatter)
+        self.logger.addHandler(self.stdout_logger)
 
         if self.file_log or "FILE_LOG" in os.environ:
             file = os.environ["FILE_LOG"] if "FILE_LOG" in os.environ else self.file_log
@@ -95,12 +112,16 @@ class LoggingSetup:
             if not os.path.exists(directory):
                 os.makedirs(directory)
             count = int(os.environ["FILE_LOG_COUNT"]) if "FILE_LOG_COUNT" in os.environ else self.file_log_count
-            handler = TimedRotatingFileHandler(
-                file, when=self.file_log_when_rotate, interval=self.file_log_rotate_interval, backupCount=count
+            self.file_logger = TimedRotatingFileHandler(
+                file,
+                when=self.file_log_when_rotate,
+                interval=self.file_log_rotate_interval,
+                backupCount=count,
+                encoding=self.file_log_encoding
             )
-            handler.setFormatter(self.formatter)
-            handler.setLevel(self.default_file_level)
-            logger.addHandler(handler)
+            self.file_logger.setFormatter(self.formatter)
+            self.file_logger.setLevel(self.default_file_level)
+            self.logger.addHandler(self.file_logger)
 
         if self.syslog or "SYSLOG" in os.environ:
             tag = os.environ["SYSLOG_TAG"] if "SYSLOG_TAG" in os.environ else self.syslog_tag
@@ -112,13 +133,13 @@ class LoggingSetup:
                 if ":" in address:
                     parts = address.split(":")
                     address = (parts[0], parts[1])
-            handler = logging.handlers.SysLogHandler(address)
-            handler.setFormatter(FancyFormatter(
+            self.syslog_logger = logging.handlers.SysLogHandler(address)
+            self.syslog_logger.setFormatter(FancyFormatter(
                 tag + self.formatter_tagged_format[0],
                 tag + self.formatter_tagged_format[1]
             ))
-            handler.setLevel(self.default_syslog_level)
-            logger.addHandler(handler)
+            self.syslog_logger.setLevel(self.default_syslog_level)
+            self.logger.addHandler(self.syslog_logger)
 
         if self.smtp_log or "SMTP_LOG" in os.environ:
             address = os.environ["SMTP_LOG_ADDRESS"] if "SMTP_LOG_ADDRESS" in os.environ else self.smtp_log_address
@@ -138,10 +159,25 @@ class LoggingSetup:
             if not recipient:
                 recipient = self.smtp_log_recipient
 
-            handler = BufferingSMTPHandler(address, sender, recipient, self.app_name)
-            handler.setFormatter(self.formatter)
-            handler.setLevel(self.default_smtp_level)
-            logger.addHandler(handler)
+            self.smtp_logger = BufferingSMTPHandler(address, sender, recipient, self.app_name)
+            self.smtp_logger.setFormatter(self.formatter)
+            self.smtp_logger.setLevel(self.default_smtp_level)
+            self.logger.addHandler(self.smtp_logger)
+
+        if self.toggle_level_signal:
+            signal.signal(self.toggle_level_signal, self.handle_signal_level_toggle)
+
+    def handle_signal_level_toggle(self, signum, frame):
+        if self.toggle_level == logging.DEBUG:
+            self.toggle_level = logging.INFO
+            logging.info("received signal - switched logging to INFO")
+        else:
+            self.toggle_level = logging.DEBUG
+            logging.info("received signal - switched logging to DEBUG")
+
+        for logger in [self.stdout_logger, self.file_logger, self.syslog_logger]:
+            if logger:
+                logger.setLevel(self.toggle_level)
 
 
 class FancyFormatter(logging.Formatter):
@@ -247,9 +283,11 @@ class BufferingSMTPHandler(logging.Handler):
             if self.should_flush():
                 self.flush()
         except Exception as e:
-            print("BufferingSMTPHandler failed: " + str(e), file=sys.stderr)
-            traceback.print_exc(file=sys.stderr)
-            sys.stderr.flush()
+            stderr = sys.stderr if sys.stderr else sys.stdout
+            print("BufferingSMTPHandler failed: " + str(e), file=stderr)
+            traceback.print_exc(file=stderr)
+            if stderr:
+                stderr.flush()
         finally:
             self.release()
 
@@ -279,7 +317,10 @@ class BufferingSMTPHandler(logging.Handler):
 
         content = []
         for record in self.buffer:
-            content.append(self.format(record))
+            try:
+                content.append(self.format(record))
+            except Exception:
+                content.append("FAILED TO FORMAT: %s" % json.dumps(record))
         message.set_content("\n".join(content))
 
         host, port = self.get_host_port()
