@@ -10,11 +10,14 @@ class SafeThread(Thread):
 
     def run(self):
         try:
-            super().run()
+            self.run_safely()
         except:
             self.crashed = "%s: %s" % (sys.exc_info()[0].__name__, sys.exc_info()[1])
             logging.exception(sys.exc_info()[0])
 
+    def run_safely(self):
+        super().run()
+
 
 class ThreadsWatcher:
     running = True
